#include <vtkSmartPointer.h>
#include <vtkPolyDataMapper.h>
#include <vtkActor.h>
#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkRenderWindowInteractor.h>

#include <vtkLookupTable.h>
#include <vtkFloatArray.h>
#include <vtkCellData.h>
#include <vtkPolyData.h>
#include <vtkPlaneSource.h>

#include <vtkPolyDataWriter.h>
#include <vtkSmartPointer.h>

#include <vtkNamedColors.h>
#include <algorithm>

template <typename T>
void PrintColour(T &rgb)
{
  // Don't do this in real code! Range checking etc. is needed.
  for (size_t i = 0; i < 3; ++i)
    {
      if (i < 2)
        {
        std::cout << static_cast<double>(rgb[i]) << " ";
        }
      else
        {
        std::cout << static_cast<double>(rgb[i]);
        }
    }
}

int main(int , char *[])
{
  // Provide some geometry
  int resolution = 3;
  vtkSmartPointer<vtkPlaneSource> aPlane =
    vtkSmartPointer<vtkPlaneSource>::New();
  aPlane->SetXResolution(resolution);
  aPlane->SetYResolution(resolution);

  // Create a lookup table to map cell data to colors
  vtkSmartPointer<vtkLookupTable> lut =
    vtkSmartPointer<vtkLookupTable>::New();
  int tableSize = std::max(resolution*resolution + 1, 10);
  lut->SetNumberOfTableValues(tableSize);
  lut->Build();

  vtkSmartPointer<vtkNamedColors> nc =
    vtkSmartPointer<vtkNamedColors>::New();

  // Fill in a few known colors, the rest will be generated if needed
  lut->SetTableValue(0, nc->GetColor4d("Black").GetData());
  lut->SetTableValue(1, nc->GetColor4d("Banana").GetData());
  lut->SetTableValue(2, nc->GetColor4d("Tomato").GetData());
  lut->SetTableValue(3, nc->GetColor4d("Wheat").GetData());
  lut->SetTableValue(4, nc->GetColor4d("Lavender").GetData());
  lut->SetTableValue(5, nc->GetColor4d("Flesh").GetData());
  lut->SetTableValue(6, nc->GetColor4d("Raspberry").GetData());
  lut->SetTableValue(7, nc->GetColor4d("Salmon").GetData());
  lut->SetTableValue(8, nc->GetColor4d("Mint").GetData());
  lut->SetTableValue(9, nc->GetColor4d("Peacock").GetData());

  aPlane->Update(); // Force an update so we can set cell data

  // Now create the cell data using the colours from the
  // lookup table. In order to do this we create an unsigned
  // char array.
  // Then use SetScalars() to add it to the vtkPolyData structure,
  // this will then be interpreted as a colour table.
  vtkSmartPointer<vtkUnsignedCharArray> colorData =
    vtkSmartPointer<vtkUnsignedCharArray>::New();
  colorData->SetName("colors"); // Any name will work here.
  colorData->SetNumberOfComponents(3);
  for (size_t i = 0; i < resolution * resolution; i++)
    {
    double rgb[3];
    unsigned char ucrgb[3];
    // Get the interpolated colour based on the cell id + 1.
    lut->GetColor(static_cast<double>((i + 1)) / (tableSize - 1), rgb);
    for (size_t j = 0; j < 3; ++j)
      {
      ucrgb[j] = static_cast<unsigned char>(rgb[j] * 255);
      }
    colorData->InsertNextTuple3(ucrgb[0], ucrgb[1], ucrgb[2]);
    // Print out what we have (look at the cell data in aa.vtk
    // to confirm that the colours have been written out).
    std::cout << "(";
    PrintColour<double[3]>(rgb);
    //std::cout << ") (";
    //PrintColour<unsigned char[3]>(ucrgb);
    std::cout << ")" << std::endl;
    }
  aPlane->GetOutput()->GetCellData()->SetScalars(colorData);

  // Setup actor and mapper
  vtkSmartPointer<vtkPolyDataMapper> mapper =
    vtkSmartPointer<vtkPolyDataMapper>::New();
  mapper->SetInputConnection(aPlane->GetOutputPort());
  mapper->SetScalarRange(0, tableSize - 1);
  mapper->SetLookupTable(lut);

  mapper->Update();

  vtkSmartPointer<vtkPolyDataWriter> writer =
    vtkSmartPointer<vtkPolyDataWriter>::New();
  writer->SetFileName( "aa.vtk" );
  writer->SetInputData( mapper->GetInput() );
  writer->Write();

  vtkSmartPointer<vtkActor> actor =
    vtkSmartPointer<vtkActor>::New();
  actor->SetMapper(mapper);

  // Setup render window, renderer, and interactor
  vtkSmartPointer<vtkRenderer> renderer =
    vtkSmartPointer<vtkRenderer>::New();
  vtkSmartPointer<vtkRenderWindow> renderWindow =
    vtkSmartPointer<vtkRenderWindow>::New();
  renderWindow->AddRenderer(renderer);
  vtkSmartPointer<vtkRenderWindowInteractor> renderWindowInteractor =
      vtkSmartPointer<vtkRenderWindowInteractor>::New();
  renderWindowInteractor->SetRenderWindow(renderWindow);
  renderer->AddActor(actor);
  renderer->SetBackground(.1,.2,.3);
  renderWindow->Render();
  renderWindowInteractor->Start();

  return EXIT_SUCCESS;
}
